<?php
/**
 * R.E. DBObjects
 *
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2010, Takács Ákos
 * @version 2.1
 * @license http://www.gnu.org/licenses/lgpl.html
 * @package REDBObjects
 */

/**
 * ADBClass absztrakt ős
 *
 * Absztrakt őse Az IIsDBListClass és az IIsDBCLass interfészeket implementáló osztályoknak.<br />
 * Ezen tulajdonságokat és metódusokat automatikusan tartalmazzák a listát,
 * illetve rekordot megvalósító osztályok
 *
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2010, Takács Ákos
 * @license http://www.gnu.org/licenses/lgpl.html
 * @package REDBObjects
 */
abstract class ADBClass
{
	/**
	* A lekérdezhető mezők asszociatív tömbje
	* @var array $properties
	*/

	protected $properties = array();

	/**
	* A megvalósítandó táblák listája
	*
	* @var array $tablelist
	*/
	public $tablelist=array();

	/**
	 * Táblák alias-ai a kulcsok, és az eredeti tábla az érték
	 *
	 * @var array $tableAliases
	 */
	public $tableAliases = array();

	/**
	 * Táblákat összekapcsoló mező neve
	 *
	 * @var string $keyName
	 */
	public $keyName="";

	/**
	 * Elsődleges kulcs mező neve
	 *
	 * Ezt a tulajdonságot szintén csak az {@link IIsDBListClass} interfészt
	 * implementáló osztályok használják fel.<br />
	 * Manuálisan sosem kell beállítani
	 *
	 * @var string $primary_key
	 */
	public $primary_key='';

	/**
	 * Táblanevet jelző string
	 *
	 * Amennyiben egyező nevű mezők vannak több táblában, jelölni kell a táblanevet is.<br />
	 * Annak jelölése, hogy táblanév is szerepel a tulajdonságban, ezzel a string-el történik.
	 *
	 * Például: 
	 * <code>
	 * $object->T_tablename_fieldname = $value;
	 * </code>
	 *
	 * @var string $tableName_signal
	 */
	protected $tableName_signal = 'T_';

	/**
	 * Táblanevet és mezőnevet elválasztó string
	 *
	 * Ha megadjuk a táblanevet is a tulajdonságokban, akkor ezzel a stringgel
	 * kell jelölni a táblanév végét (Mező kezdetét)
	 * Például: 
	 * <code>
	 * $object->T_tablename_fieldname = $value;
	 * </code>
	 *
	 * @var string $table_field_sep
	 */
	protected $table_field_sep = '_';

	/**
	 * A táblák elsődleges kulcs mezőinek nevét tároló tömb.
	 *
	 * Asszociatív kétdimenziós tömb.
	 * 1. dimenzió kulcsa a tábla neve.
	 * 2. dimenzió kulcsa a kulcsmező neve, értéke a kulcsmez tartalma.
	 *
	 * @var array $priKeys
	 */
	protected $priKeys=array();

	/**
	 * Tábla és mezőnevet szétválasztó metódus
	 *
	 * A tábla és mezőnevek a nekik megfelelő referencia változókba kerülnek
	 * a művelet elvégzése után
	 *
	 * @param string $input A tábla és mezőnév egy tulajdonságnévként
	 * @param string $table Táblanév (referencia változó)
	 * @param string $field Mezőnév (referencia változó)
	 * @return bool true, ha táblanévvel volt hívva a tulajdonság, és false ha nem
	 */
	protected function sep_table_field($input,&$table,&$field)
	{
		if(preg_match('/^('.$this->tableName_signal.')(.*)('.$this->table_field_sep.')(.*)$/',$input,$out)) {
			$table = $out[2];
			$field = $out[4];
			return true;
		}
		return false;
	}
}
?>
